@@ -1,11 +1,11 @@
 from itertools import groupby
-from typing import Optional
+from typing import Optional, Iterable
 
 from dfa import dict2dfa, DFA
 from pysat.solvers import Glucose4
 
 from dfa_identify.graphs import Word, APTA
-from dfa_identify.encoding import dfa_id_encodings, Codec
+from dfa_identify.encoding import dfa_id_encodings, Codec, SymMode
 from dfa_identify.encoding import (
     ColorAcceptingVar,
     ColorNodeVar,
@@ -52,10 +52,44 @@ def extract_dfa(codec: Codec, apta: APTA, model: list[int]) -> DFA:
     return dict2dfa(dfa_dict, start=node2color[0])
 
 
+def find_dfas(
+        accepting: list[Word],
+        rejecting: list[Word],
+        solver_fact=Glucose4,
+        sym_mode: SymMode = "bfs",
+) -> Iterable[DFA]:
+    """Finds all minimal dfa that are consistent with the labeled examples.
+
+    Here "minimal" means that a no DFA with smaller size is consistent with
+    the data. Thus, all returns DFAs are the same size.
+
+    Inputs:
+      - accepting: A sequence of "words" to be accepted.
+      - rejecting: A sequence of "words" to be rejected.
+      - solver: A py-sat API compatible object for solving CNF SAT queries.
+
+    Returns:
+      An iterable of all minimal DFA consistent with accepting and rejecting.
+    """
+    apta = APTA.from_examples(accepting=accepting, rejecting=rejecting)
+    for codec, clauses in dfa_id_encodings(apta, sym_mode=sym_mode):
+        with solver_fact() as solver:
+            for clause in clauses:
+                solver.add_clause(clause)
+
+            if not solver.solve():
+                continue
+
+            models = solver.enum_models()
+            yield from (extract_dfa(codec, apta, model) for model in models)
+            return
+
+
 def find_dfa(
         accepting: list[Word],
         rejecting: list[Word],
         solver_fact=Glucose4,
+        sym_mode: SymMode = "clique",
 ) -> Optional[DFA]:
     """Finds a minimal dfa that is consistent with the labeled examples.
 
@@ -68,16 +102,7 @@ def find_dfa(
       Either a DFA consistent with accepting and rejecting or None
       indicating that no DFA exists.
     """
-
-    apta = APTA.from_examples(accepting=accepting, rejecting=rejecting)
-    for codec, clauses in dfa_id_encodings(apta):
-        with solver_fact() as solver:
-            for clause in clauses:
-                solver.add_clause(clause)
-
-            if solver.solve():
-                model = solver.get_model()
-                return extract_dfa(codec, apta, model)
+    return next(find_dfas(accepting, rejecting, solver_fact, sym_mode), None)
 
 
-__all__ = ['find_dfa', 'extract_dfa']
+__all__ = ['find_dfas', 'find_dfa', 'extract_dfa']
